--[[
	ChangeEnterText
	Specialization text to enter a vehicle
	
	@author:  	Ifko[nator]
	@date:  	15.03.2019
	@version:	1.0
]]

ChangeEnterText = {};

function ChangeEnterText.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Enterable, specializations);
end;

function ChangeEnterText.registerEventListeners(vehicleType)
	local functionNames = {
        "onLoad"
	};

	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, ChangeEnterText);
	end;
end;

function ChangeEnterText:onLoad(savegame) 
    local specEnterable = self.spec_enterable;
     
    specEnterable.text = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.enterable#enterText"), "button_enterVehicle");

    Player.updateActionEvents = Utils.overwrittenFunction(Player.updateActionEvents, ChangeEnterText.updateActionEvents)
end;

function ChangeEnterText:updateActionEvents()
    -- light
    local isDark = g_currentMission.environment.currentHour > g_currentMission.environment.nightStart - 0.5 or g_currentMission.environment.currentHour < g_currentMission.environment.nightEnd + 0.5;
	local eventIdToggleLight = self.inputInformation.registrationList[InputAction.TOGGLE_LIGHTS_FPS].eventId;
	
    if self.playerStateMachine:isAvailable("useLight") and isDark then
        g_inputBinding:setActionEventTextVisibility(eventIdToggleLight, isDark and self.lightNode ~= nil);
	end;
	
	local eventList = self.inputInformation.registrationList;
	
	function disableInput(inputAction)
		local id = eventList[inputAction].eventId;
		
        g_inputBinding:setActionEventActive(id, false);
        g_inputBinding:setActionEventTextVisibility(id, false);
	end;
	
    function enableInput(inputAction)
        local id = eventList[inputAction].eventId;
        g_inputBinding:setActionEventActive(id, true);
        g_inputBinding:setActionEventTextVisibility(id, true);
	end;
	
    disableInput(InputAction.NEXT_HANDTOOL);
    disableInput(InputAction.PREVIOUS_HANDTOOL);
    disableInput(InputAction.ACTIVATE_HANDTOOL);
    disableInput(InputAction.INTERACT);
    disableInput(InputAction.ACTIVATE_OBJECT);
    disableInput(InputAction.ENTER);
    disableInput(InputAction.THROW_OBJECT);
    disableInput(InputAction.ROTATE_OBJECT_LEFT_RIGHT);
	disableInput(InputAction.ROTATE_OBJECT_UP_DOWN);
	
    -- tools
    if self.playerStateMachine:isAvailable("cycleHandtool") then
        enableInput(InputAction.NEXT_HANDTOOL);
        enableInput(InputAction.PREVIOUS_HANDTOOL);
	end;
	
    if self:hasHandtoolEquipped() then
		enableInput(InputAction.ACTIVATE_HANDTOOL);
		
		local eventIdActivateObject = self.inputInformation.registrationList[InputAction.ACTIVATE_OBJECT].eventId;
		
        if self.playerStateMachine:isAvailable("activateObject") then
			enableInput(InputAction.ACTIVATE_OBJECT);
			
			local activateObjectState = self.playerStateMachine:getState("activateObject");
			
            g_inputBinding:setActionEventText(eventIdActivateObject, activateObjectState.activateText);
        end;
    else
        if self.playerStateMachine:isAvailable("throw") then
            enableInput(InputAction.THROW_OBJECT);
		end;
		
        local eventIdObjectRotateHorizontally = self.inputInformation.registrationList[InputAction.ROTATE_OBJECT_LEFT_RIGHT].eventId;
		local eventIdObjectRotateVertically = self.inputInformation.registrationList[InputAction.ROTATE_OBJECT_UP_DOWN].eventId;
		
        if self.isCarryingObject then
            enableInput(InputAction.ROTATE_OBJECT_LEFT_RIGHT);
            enableInput(InputAction.ROTATE_OBJECT_UP_DOWN);
		end;
		
		local eventIdActivateObject = self.inputInformation.registrationList[InputAction.ACTIVATE_OBJECT].eventId;
		
        if self.playerStateMachine:isAvailable("activateObject") then
			enableInput(InputAction.ACTIVATE_OBJECT);
			
			local activateObjectState = self.playerStateMachine:getState("activateObject");
			
            g_inputBinding:setActionEventText(eventIdActivateObject, activateObjectState.activateText);
        elseif self.playerStateMachine:isAvailable("animalFeed") then
			enableInput(InputAction.ACTIVATE_OBJECT);
			
            g_inputBinding:setActionEventText(eventIdActivateObject, g_i18n:getText("action_feedAnimal"));
        elseif self.playerStateMachine:isAvailable("animalPet") then
			enableInput(InputAction.ACTIVATE_OBJECT);
			
            g_inputBinding:setActionEventText(eventIdActivateObject, g_i18n:getText("action_petAnimal"));
		end;
		
		local eventIdInteract = self.inputInformation.registrationList[InputAction.INTERACT].eventId;
		
        if self.playerStateMachine:isAvailable("drop") then
			g_inputBinding:setActionEventText(eventIdInteract, g_i18n:getText("action_dropObject"));
			
            enableInput(InputAction.INTERACT);
        elseif self.playerStateMachine:isAvailable("pickup") then
			g_inputBinding:setActionEventText(eventIdInteract, g_i18n:getText("action_pickUpObject"));
			
            enableInput(InputAction.INTERACT);
        elseif self.playerStateMachine:isAvailable("animalInteract") or self.playerStateMachine:isActive("animalInteract") then
            local animalInteractState = self.playerStateMachine:getState("animalInteract");
			local animalInteractText = string.format(g_i18n:getText("action_interactAnimal"), animalInteractState.interactText);
			
			g_inputBinding:setActionEventText(eventIdInteract, animalInteractText);
			
            enableInput(InputAction.INTERACT);
        elseif self.inputInformation.interactState == Player.BUTTONSTATES.RELEASED then
            disableInput(InputAction.INTERACT);
        end;
	end;
	
    -- enter vehicle or ride animal
    self.canRideAnimal = self.playerStateMachine:isAvailable("animalRide");
	self.canEnterVehicle = g_currentMission.interactiveVehicleInRange and g_currentMission.interactiveVehicleInRange:getIsEnterable();
	
	local vehicleIsRideable = self.canEnterVehicle and SpecializationUtil.hasSpecialization(Rideable, g_currentMission.interactiveVehicleInRange.specializations);
	local vehicleNeedChangeEnterText = self.canEnterVehicle and SpecializationUtil.hasSpecialization(ChangeEnterText, g_currentMission.interactiveVehicleInRange.specializations);
	local eventIdEnter = self.inputInformation.registrationList[InputAction.ENTER].eventId;
	
    if self.canEnterVehicle and not vehicleIsRideable then
		if vehicleNeedChangeEnterText then
			local specEnterable = g_currentMission.interactiveVehicleInRange.spec_enterable;
			local text = specEnterable.text;

			if text:sub(1, 6) == "$l10n_" then
				text = g_i18n:getText(text:sub(7));
			elseif g_i18n:hasText(text) then
				text = g_i18n:getText(text);
			end;

			g_inputBinding:setActionEventText(eventIdEnter, text);
		else
			g_inputBinding:setActionEventText(eventIdEnter, g_i18n:getText("button_enterVehicle"))
		end;

        enableInput(InputAction.ENTER);
    elseif self.canRideAnimal or vehicleIsRideable then
		local rideableName = "";
		
        if self.canRideAnimal then
			local rideState = self.playerStateMachine:getState("animalRide");
			
            rideableName = rideState:getRideableName();
        elseif vehicleIsRideable then
            rideableName = g_currentMission.interactiveVehicleInRange:getFullName();
		end
		
		g_inputBinding:setActionEventText(eventIdEnter, string.format(g_i18n:getText("action_rideAnimal"), rideableName));
		
        enableInput(InputAction.ENTER);
	end;
	
    -- debug movements
	local eventIdDebugFlyToggle = self.inputInformation.registrationList[InputAction.DEBUG_PLAYER_ENABLE].eventId;
	local eventIdDebugFlyUpDown = self.inputInformation.registrationList[InputAction.DEBUG_PLAYER_UP_DOWN].eventId;

	g_inputBinding:setActionEventActive(eventIdDebugFlyToggle, g_flightAndNoHUDKeysEnabled);
    g_inputBinding:setActionEventActive(eventIdDebugFlyUpDown, g_flightAndNoHUDKeysEnabled);
end;