--[[
	RenderElement
	Specialization for to render overlays and texts at game start
	
	@author:  	Ifko[nator]
	@date:  	23.05.2019
	@version:	1.0
]]

RenderElement = {};
RenderElement.currentModDirectory = g_currentModDirectory;

function RenderElement:loadMap(name)
	local modDesc = loadXMLFile("modDesc", RenderElement.currentModDirectory .. "modDesc.xml");

	self.renderElements = {};
	
	local renderElementCount = 0;
	
	while true do
		local renderElementKey = string.format("modDesc.renderElements.renderElement(%d)", renderElementCount);
		
		if not hasXMLProperty(modDesc, renderElementKey) then
			break;
		end;
		
		local renderElement = {};

		renderElement.textPosX = Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. ".text#posX"), 0.5);
		renderElement.textPosY = Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. ".text#posY"), 0.015);
		renderElement.textSize = Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. ".text#size"), 0.02);
		renderElement.textName = Utils.getNoNil(getXMLString(modDesc, renderElementKey .. ".text#name"), "");
		renderElement.useTextBold = Utils.getNoNil(getXMLBool(modDesc, renderElementKey .. ".text#useBold"), true);
		renderElement.useTextColoring = Utils.getNoNil(getXMLBool(modDesc, renderElementKey .. ".text#useColoring"), false);
		renderElement.useTextSizeing = Utils.getNoNil(getXMLBool(modDesc, renderElementKey .. ".text#useSizeing"), false);
		renderElement.minTextSize = renderElement.textSize;
		renderElement.maxTextSize = Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. ".text#maxSize"), 0.03);
		renderElement.wasMaxSize = false;
		renderElement.hasSetNewTextColor = false;
		renderElement.textColor = Utils.getNoNil(StringUtil.getVectorNFromString(getXMLString(modDesc, renderElementKey .. ".text#color"), 3), {1, 1, 1}); 
		
		if renderElement.textName:sub(1, 6) == "$l10n_" then
			renderElement.textName = g_i18n:getText(renderElement.textName:sub(7));
		end;
		
		renderElement.showOnlyOnFirstStart = Utils.getNoNil(getXMLBool(modDesc, renderElementKey .. "#showOnlyOnFirstStart"), true);
		renderElement.startTime = Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. "#startTime"), 0);
		renderElement.endTime = Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. "#endTime"), 400);
		renderElement.time = 0;
		
		if renderElementCount == 0 then
			renderElement.alpha = 1;
		else
			renderElement.alpha = 0;
		end;
		
		local filename = getXMLString(modDesc, renderElementKey .. ".overlay#filename");
		
		if filename ~= nil then		
			renderElement.overlay = Overlay:new(Utils.getFilename(filename, RenderElement.currentModDirectory), 
				Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. ".overlay#posX"), 0.25), 
				Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. ".overlay#posY"), 0), 
				Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. ".overlay#width"), 0.15), 
				Utils.getNoNil(getXMLFloat(modDesc, renderElementKey .. ".overlay#height"), 0.1)
			);
		end;
		
		table.insert(self.renderElements, renderElement);
		
		renderElementCount = renderElementCount + 1;
	end;
	
	delete(modDesc);
end;

function RenderElement:draw()
 	if g_gui.currentGui ~= nil or g_currentMission.isPlayerFrozen then
		return;
	end;
	
	if self.renderElements ~= nil then
		setTextAlignment(RenderText.ALIGN_CENTER);

		for renderElementNumber = 1, #self.renderElements do
			local renderElement = self.renderElements[renderElementNumber];

			if (renderElement.showOnlyOnFirstStart and not g_currentMission.missionInfo.isValid) or not renderElement.showOnlyOnFirstStart then
				renderElement.time = renderElement.time + 1;

				if renderElement.startTime <= renderElement.time then
					if renderElement.time > renderElement.endTime then
						renderElement.alpha = math.max(renderElement.alpha - 0.01, 0);
					else
						renderElement.alpha = math.min(renderElement.alpha + 0.01, 1);
					end;

					if renderElement.alpha > 0 then
						if renderElement.overlay ~= nil then
							renderElement.overlay:setColor(nil, nil, nil, renderElement.alpha);

							renderElement.overlay:render(renderElement.alpha > 0.01);
						end;

						if renderElement.textName ~= "" then
							if renderElement.useTextColoring then
								if renderElement.textColor[1] < 1 and renderElement.textColor[2] < 1  and renderElement.textColor[3] < 1 and not renderElement.hasSetNewTextColor then	
									renderElement.textColor[1] = math.min(renderElement.textColor[1] + 0.15, 1);
									renderElement.textColor[2] = math.min(renderElement.textColor[2] + 0.15, 1);
									renderElement.textColor[3] = math.min(renderElement.textColor[3] + 0.15, 1);

									renderElement.hasSetNewTextColor = renderElement.textColor[1] >= 1 and renderElement.textColor[2] >= 1  and renderElement.textColor[3] >= 1;
								else
									renderElement.textColor[1] = math.max(renderElement.textColor[1] - 0.15, 0);
									renderElement.textColor[2] = math.max(renderElement.textColor[2] - 0.15, 0);
									renderElement.textColor[3] = math.max(renderElement.textColor[3] - 0.15, 0);

									if renderElement.textColor[1] <= 0 and renderElement.textColor[2] <= 0 and renderElement.textColor[3] <= 0 then
										renderElement.hasSetNewTextColor = false;
									end;
								end;
							end;

							if renderElement.useTextSizeing then
								if renderElement.textSize < renderElement.maxTextSize and not renderElement.wasMaxSize then
									renderElement.textSize = math.min(renderElement.textSize + 0.0007, renderElement.maxTextSize);
								
									renderElement.wasMaxSize = renderElement.textSize >= renderElement.maxTextSize;
								elseif renderElement.wasMaxSize then
									renderElement.textSize = math.max(renderElement.textSize - 0.0007, renderElement.minTextSize);

									if renderElement.textSize <= renderElement.minTextSize then
										renderElement.wasMaxSize = false;
									end;
								end;
							end;

							setTextBold(renderElement.useTextBold);
							setTextColor(renderElement.textColor[1], renderElement.textColor[2], renderElement.textColor[3], renderElement.alpha);
						
							renderText(renderElement.textPosX, renderElement.textPosY, getCorrectTextSize(renderElement.textSize), renderElement.textName);
						end;
					end;
				end;
			end;
		end;

		setTextColor(1.0, 1.0, 1.0, 1.0);
		setTextBold(false);
		setTextAlignment(RenderText.ALIGN_LEFT);
	end;
end;

function RenderElement:deleteMap()end;
function RenderElement:keyEvent(unicode, sym, modifier, isDown)end;
function RenderElement:mouseEvent(posX, posY, isDown, isUp, button)end;
function RenderElement:update(dt)end;

addModEventListener(RenderElement);